==> mturk-2.1.0-sources.jar: software/amazon/awssdk/services/mturk/paginators/ListAssignmentsForHITPublisher.java <==
/*
 * Copyright 2013-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
 * the License. A copy of the License is located at
 * 
 * http://aws.amazon.com/apache2.0
 * 
 * or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */

package software.amazon.awssdk.services.mturk.paginators;

import java.util.concurrent.CompletableFuture;
import org.reactivestreams.Subscriber;
import software.amazon.awssdk.annotations.Generated;
import software.amazon.awssdk.core.async.SdkPublisher;
import software.amazon.awssdk.core.pagination.async.AsyncPageFetcher;
import software.amazon.awssdk.core.pagination.async.EmptySubscription;
import software.amazon.awssdk.core.pagination.async.ResponsesSubscription;
import software.amazon.awssdk.core.util.PaginatorUtils;
import software.amazon.awssdk.services.mturk.MTurkAsyncClient;
import software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitRequest;
import software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitResponse;

/**
 * <p>
 * Represents the output for the
 * {@link software.amazon.awssdk.services.mturk.MTurkAsyncClient#listAssignmentsForHITPaginator(software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitRequest)}
 * operation which is a paginated operation. This class is a type of {@link org.reactivestreams.Publisher} which can be
 * used to provide a sequence of {@link software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitResponse}
 * response pages as per demand from the subscriber.
 * </p>
 * <p>
 * When the operation is called, an instance of this class is returned. At this point, no service calls are made yet and
 * so there is no guarantee that the request is valid. If there are errors in your request, you will see the failures
 * only after you start streaming the data. The subscribe method should be called as a request to start streaming data.
 * For more info, see {@link org.reactivestreams.Publisher#subscribe(org.reactivestreams.Subscriber)}. Each call to the
 * subscribe method will result in a new {@link org.reactivestreams.Subscription} i.e., a new contract to stream data
 * from the starting request.
 * </p>
 *
 * <p>
 * The following are few ways to use the response class:
 * </p>
 * 1) Using the forEach helper method
 * 
 * <pre>
 * {@code
 * software.amazon.awssdk.services.mturk.paginators.ListAssignmentsForHITPublisher publisher = client.listAssignmentsForHITPaginator(request);
 * CompletableFuture<Void> future = publisher.forEach(res -> { // Do something with the response });
 * future.get();
 * }
 * </pre>
 *
 * 2) Using a custom subscriber
 * 
 * <pre>
 * {@code
 * software.amazon.awssdk.services.mturk.paginators.ListAssignmentsForHITPublisher publisher = client.listAssignmentsForHITPaginator(request);
 * publisher.subscribe(new Subscriber<software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitResponse>() {
 * 
 * public void onSubscribe(org.reactivestreams.Subscriber subscription) { //... };
 * 
 * 
 * public void onNext(software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitResponse response) { //... };
 * });}
 * </pre>
 * 
 * As the response is a publisher, it can work well with third party reactive streams implementations like RxJava2.
 * <p>
 * <b>Note: If you prefer to have control on service calls, use the
 * {@link #listAssignmentsForHIT(software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitRequest)}
 * operation.</b>
 * </p>
 */
@Generated("software.amazon.awssdk:codegen")
public class ListAssignmentsForHITPublisher implements SdkPublisher<ListAssignmentsForHitResponse> {
    private final MTurkAsyncClient client;

    private final ListAssignmentsForHitRequest firstRequest;

    private final AsyncPageFetcher nextPageFetcher;

    private boolean isLastPage;

    public ListAssignmentsForHITPublisher(MTurkAsyncClient client, ListAssignmentsForHitRequest firstRequest) {
        this(client, firstRequest, false);
    }

    private ListAssignmentsForHITPublisher(MTurkAsyncClient client, ListAssignmentsForHitRequest firstRequest, boolean isLastPage) {
        this.client = client;
        this.firstRequest = firstRequest;
        this.isLastPage = isLastPage;
        this.nextPageFetcher = new ListAssignmentsForHitResponseFetcher();
    }

    @Override
    public void subscribe(Subscriber<? super ListAssignmentsForHitResponse> subscriber) {
        subscriber.onSubscribe(ResponsesSubscription.builder().subscriber(subscriber).nextPageFetcher(nextPageFetcher).build());
    }

    /**
     * <p>
     * A helper method to resume the pages in case of unexpected failures. The method takes the last successful response
     * page as input and returns an instance of {@link ListAssignmentsForHITPublisher} that can be used to retrieve the
     * consecutive pages that follows the input page.
     * </p>
     */
    private final ListAssignmentsForHITPublisher resume(ListAssignmentsForHitResponse lastSuccessfulPage) {
        if (nextPageFetcher.hasNextPage(lastSuccessfulPage)) {
            return new ListAssignmentsForHITPublisher(client, firstRequest.toBuilder().nextToken(lastSuccessfulPage.nextToken())
                    .build());
        }
        return new ListAssignmentsForHITPublisher(client, firstRequest, true) {
            @Override
            public void subscribe(Subscriber<? super ListAssignmentsForHitResponse> subscriber) {
                subscriber.onSubscribe(new EmptySubscription(subscriber));
            }
        };
    }

    private class ListAssignmentsForHitResponseFetcher implements AsyncPageFetcher<ListAssignmentsForHitResponse> {
        @Override
        public boolean hasNextPage(final ListAssignmentsForHitResponse previousPage) {
            return PaginatorUtils.isOutputTokenAvailable(previousPage.nextToken());
        }

        @Override
        public CompletableFuture<ListAssignmentsForHitResponse> nextPage(final ListAssignmentsForHitResponse previousPage) {
            if (previousPage == null) {
                return client.listAssignmentsForHIT(firstRequest);
            }
            return client.listAssignmentsForHIT(firstRequest.toBuilder().nextToken(previousPage.nextToken()).build());
        }
    }
}

==> mturk-2.1.2-sources.jar: software/amazon/awssdk/services/mturk/paginators/ListAssignmentsForHITPublisher.java <==
/*
 * Copyright 2013-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
 * the License. A copy of the License is located at
 * 
 * http://aws.amazon.com/apache2.0
 * 
 * or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */

package software.amazon.awssdk.services.mturk.paginators;

import java.util.concurrent.CompletableFuture;
import org.reactivestreams.Subscriber;
import software.amazon.awssdk.annotations.Generated;
import software.amazon.awssdk.core.async.SdkPublisher;
import software.amazon.awssdk.core.pagination.async.AsyncPageFetcher;
import software.amazon.awssdk.core.pagination.async.EmptySubscription;
import software.amazon.awssdk.core.pagination.async.ResponsesSubscription;
import software.amazon.awssdk.core.util.PaginatorUtils;
import software.amazon.awssdk.services.mturk.MTurkAsyncClient;
import software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitRequest;
import software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitResponse;

/**
 * <p>
 * Represents the output for the
 * {@link software.amazon.awssdk.services.mturk.MTurkAsyncClient#listAssignmentsForHITPaginator(software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitRequest)}
 * operation which is a paginated operation. This class is a type of {@link org.reactivestreams.Publisher} which can be
 * used to provide a sequence of {@link software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitResponse}
 * response pages as per demand from the subscriber.
 * </p>
 * <p>
 * When the operation is called, an instance of this class is returned. At this point, no service calls are made yet and
 * so there is no guarantee that the request is valid. If there are errors in your request, you will see the failures
 * only after you start streaming the data. The subscribe method should be called as a request to start streaming data.
 * For more info, see {@link org.reactivestreams.Publisher#subscribe(org.reactivestreams.Subscriber)}. Each call to the
 * subscribe method will result in a new {@link org.reactivestreams.Subscription} i.e., a new contract to stream data
 * from the starting request.
 * </p>
 *
 * <p>
 * The following are few ways to use the response class:
 * </p>
 * 1) Using the subscribe helper method
 * 
 * <pre>
 * {@code
 * software.amazon.awssdk.services.mturk.paginators.ListAssignmentsForHITPublisher publisher = client.listAssignmentsForHITPaginator(request);
 * CompletableFuture<Void> future = publisher.subscribe(res -> { // Do something with the response });
 * future.get();
 * }
 * </pre>
 *
 * 2) Using a custom subscriber
 * 
 * <pre>
 * {@code
 * software.amazon.awssdk.services.mturk.paginators.ListAssignmentsForHITPublisher publisher = client.listAssignmentsForHITPaginator(request);
 * publisher.subscribe(new Subscriber<software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitResponse>() {
 * 
 * public void onSubscribe(org.reactivestreams.Subscriber subscription) { //... };
 * 
 * 
 * public void onNext(software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitResponse response) { //... };
 * });}
 * </pre>
 * 
 * As the response is a publisher, it can work well with third party reactive streams implementations like RxJava2.
 * <p>
 * <b>Note: If you prefer to have control on service calls, use the
 * {@link #listAssignmentsForHIT(software.amazon.awssdk.services.mturk.model.ListAssignmentsForHitRequest)}
 * operation.</b>
 * </p>
 */
@Generated("software.amazon.awssdk:codegen")
public class ListAssignmentsForHITPublisher implements SdkPublisher<ListAssignmentsForHitResponse> {
    private final MTurkAsyncClient client;

    private final ListAssignmentsForHitRequest firstRequest;

    private final AsyncPageFetcher nextPageFetcher;

    private boolean isLastPage;

    public ListAssignmentsForHITPublisher(MTurkAsyncClient client, ListAssignmentsForHitRequest firstRequest) {
        this(client, firstRequest, false);
    }

    private ListAssignmentsForHITPublisher(MTurkAsyncClient client, ListAssignmentsForHitRequest firstRequest, boolean isLastPage) {
        this.client = client;
        this.firstRequest = firstRequest;
        this.isLastPage = isLastPage;
        this.nextPageFetcher = new ListAssignmentsForHitResponseFetcher();
    }

    @Override
    public void subscribe(Subscriber<? super ListAssignmentsForHitResponse> subscriber) {
        subscriber.onSubscribe(ResponsesSubscription.builder().subscriber(subscriber).nextPageFetcher(nextPageFetcher).build());
    }

    /**
     * <p>
     * A helper method to resume the pages in case of unexpected failures. The method takes the last successful response
     * page as input and returns an instance of {@link ListAssignmentsForHITPublisher} that can be used to retrieve the
     * consecutive pages that follows the input page.
     * </p>
     */
    private final ListAssignmentsForHITPublisher resume(ListAssignmentsForHitResponse lastSuccessfulPage) {
        if (nextPageFetcher.hasNextPage(lastSuccessfulPage)) {
            return new ListAssignmentsForHITPublisher(client, firstRequest.toBuilder().nextToken(lastSuccessfulPage.nextToken())
                    .build());
        }
        return new ListAssignmentsForHITPublisher(client, firstRequest, true) {
            @Override
            public void subscribe(Subscriber<? super ListAssignmentsForHitResponse> subscriber) {
                subscriber.onSubscribe(new EmptySubscription(subscriber));
            }
        };
    }

    private class ListAssignmentsForHitResponseFetcher implements AsyncPageFetcher<ListAssignmentsForHitResponse> {
        @Override
        public boolean hasNextPage(final ListAssignmentsForHitResponse previousPage) {
            return PaginatorUtils.isOutputTokenAvailable(previousPage.nextToken());
        }

        @Override
        public CompletableFuture<ListAssignmentsForHitResponse> nextPage(final ListAssignmentsForHitResponse previousPage) {
            if (previousPage == null) {
                return client.listAssignmentsForHIT(firstRequest);
            }
            return client.listAssignmentsForHIT(firstRequest.toBuilder().nextToken(previousPage.nextToken()).build());
        }
    }
}
